package newtonreferat;

import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.lsmp.djep.djep.DJep;
import org.nfunk.jep.Node;
import org.nfunk.jep.ParseException;

public class NewtonGUI extends JFrame {
	
	private static final long serialVersionUID = 1L;
	private static XYSeriesCollection collection = new XYSeriesCollection();
	private JFreeChart chart;
	private JPanel jContentPane = null;
	private ChartPanel chartPanel;
	private JPanel jPanel = null;
	private JTextField textFormula = null;
	private JLabel lblFormula = null;
	private JButton btnCalculate = null;
	private JButton btnClear = null;
	private int step;
	private JTextField textFrom = null;
	private JLabel lblFrom = null;
	private JLabel lblTo = null;
	private JTextField textTo = null;
	private DJep jep = new DJep();
	private JLabel lblGuess = null;
	private JTextField textGuess = null;
	private JLabel lblDerivative = null;
	private JTextField textDerivative = null;
	private double lastX;
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			lblDerivative = new JLabel();
			lblDerivative.setText("Derivative");
			lblGuess = new JLabel();
			lblGuess.setText("Guess");
			lblTo = new JLabel();
			lblTo.setText("To");
			lblFrom = new JLabel();
			lblFrom.setText("From");
			lblFormula = new JLabel();
			lblFormula.setText("Formula");
			jPanel = new JPanel();
			jPanel.setLayout(new FlowLayout());
			jPanel.add(lblFormula, null);
			jPanel.add(getTextFormula(), null);
			jPanel.add(lblDerivative, null);
			jPanel.add(getTextDerivative(), null);
			jPanel.add(lblFrom, null);
			jPanel.add(getTextFrom(), null);
			jPanel.add(lblTo, null);
			jPanel.add(getTextTo(), null);
			jPanel.add(lblGuess, null);
			jPanel.add(getTextGuess(), null);
			jPanel.add(getBtnCalculate(), null);
			jPanel.add(getBtnClear(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes textFormula	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTextFormula() {
		if (textFormula == null) {
			textFormula = new JTextField();
			textFormula.setText("x^3-5              ");
		}
		return textFormula;
	}

	/**
	 * This method initializes btnCalculate	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBtnCalculate() {
		if (btnCalculate == null) {
			btnCalculate = new JButton();
			btnCalculate.setText("Step");
			btnCalculate.addActionListener(new java.awt.event.ActionListener() {

				public void actionPerformed(java.awt.event.ActionEvent e) {
					String formula = textFormula.getText();
					String derivative = null;
					
					try {
						jep.addStandardConstants();
						jep.addStandardFunctions();
						jep.addComplex();
						jep.setAllowUndeclared(true);
						jep.setAllowAssignment(true);
						jep.setImplicitMul(true);

						// Sets up standard rules for differentiating sin(x) etc.
						jep.addStandardDiffRules();
						Node node = jep.parse(formula);
						
						Node diff = jep.differentiate(node,"x");
						diff = jep.simplify(diff);
						derivative = jep.toString(diff);
						
						textDerivative.setText(derivative);
					} catch (ParseException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
					
					if(step==0){
						lastX = Double.parseDouble(textGuess.getText());
						XYSeries functionSeries = new XYSeries("Function");
						Integer fromVal = Integer.parseInt(textFrom.getText());
						Integer toVal = Integer.parseInt(textTo.getText());
						jep.parseExpression(formula);

						for (int i = fromVal; i < toVal; i++) {
							jep.addVariable("x",i);
							functionSeries.add(i,jep.getValue());
						}
						collection.addSeries(functionSeries);
					}else{
						XYSeries newLine = new XYSeries("Tangent #"+step);
						jep.parseExpression(formula);
						jep.addVariable("x",lastX);
						
						double y1 = jep.getValue();
						newLine.add(lastX,y1);
						
						
						jep.parseExpression(formula);
						jep.addVariable("x",lastX);
						double fx1 = jep.getValue();
						
						jep.parseExpression(derivative);
						jep.addVariable("x",lastX);
						double fderiv1 = jep.getValue();
						
						double tmp = fx1/fderiv1;
						lastX = lastX-tmp;
						System.out.println(lastX);
						newLine.add(lastX,0);
						collection.addSeries(newLine);
					}
					step++;
				}
			});
		}
		return btnCalculate;
	}

	/**
	 * This method initializes btnClear	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBtnClear() {
		if (btnClear == null) {
			btnClear = new JButton();
			btnClear.setText("Clear");
			btnClear.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					step=0;
					collection.removeAllSeries();
				}
			});
		}
		return btnClear;
	}

	/**
	 * This method initializes textFrom	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTextFrom() {
		if (textFrom == null) {
			textFrom = new JTextField();
			textFrom.setText("-10");
		}
		return textFrom;
	}

	/**
	 * This method initializes textTo	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTextTo() {
		if (textTo == null) {
			textTo = new JTextField();
			textTo.setText("10");
		}
		return textTo;
	}

	/**
	 * This method initializes textGuess	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTextGuess() {
		if (textGuess == null) {
			textGuess = new JTextField();
			textGuess.setText("3     ");
		}
		return textGuess;
	}

	/**
	 * This method initializes textDerivative	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getTextDerivative() {
		if (textDerivative == null) {
			textDerivative = new JTextField();
			textDerivative.setText("3*x^2              ");
		}
		return textDerivative;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		NewtonGUI g = new NewtonGUI();
		g.pack();
		g.setVisible(true);
	}

	/**
	 * This is the default constructor
	 */
	public NewtonGUI() {
		super();
		initialize();
		jep.addStandardFunctions();
		jep.addStandardConstants();
		jep.addVariable("x", 0);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(771, 515);
		this.setContentPane(getJContentPane());
		this.setTitle("JFrame");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new java.awt.BorderLayout());
			jContentPane.add(getJPanel(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getChartPanel(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}
    private ChartPanel getChartPanel() {
        if (chartPanel == null) {
            chartPanel = new ChartPanel(getChart());
            chartPanel.setVisible(true);
            chartPanel.setPreferredSize(new Dimension(400, 400));
        }
        return chartPanel;
    }
    private JFreeChart getChart() {
        if (chart == null) {
            chart = ChartFactory.createXYLineChart("Iterationsverfahren", "X", "Y",  collection,
                    PlotOrientation.VERTICAL, true, true, false);
        }
        return chart;
    }
    
}  //  @jve:decl-index=0:visual-constraint="10,10"
