package newtonreferat;
import java.math.BigDecimal;

/**
 * $Id: PiCalc.java,v 1.1 2006/01/11 18:31:17 tag Exp $ Created: Oct 2, 2005
 * 
 * @author <a href="mailto:tag@gmx.de">Juan Antonio Agudo</a>
 */
public class PiCalc {

	public static final BigDecimal ZERO = new BigDecimal("0");
	public static final BigDecimal ONE  = new BigDecimal("1");
	public static final BigDecimal TWO  = new BigDecimal("2");

	public static void main(String[] args) {
		int precision = 512;
		int scale = precision;
		long start = System.currentTimeMillis();
		long end =0;
		BigDecimal a = ONE;
		BigDecimal b = ONE.divide(newtSqrt(TWO,scale),scale,BigDecimal.ROUND_DOWN);
		BigDecimal t = new BigDecimal("0.25");
		BigDecimal p = ONE;

		for (int i = 0; i < precision; i++) {
			BigDecimal x = a.add(b).divide(TWO);
			BigDecimal y = newtSqrt(a.multiply(b),scale/2);
			
			
			BigDecimal ax = a.subtract(x);
			ax = ax.pow(2);
			t = t.subtract(p.multiply(ax));
			a = x;
			b = y;
			p = TWO.multiply(p);
		}
		end = System.currentTimeMillis();
		System.out.println(end-start+"ms");
		BigDecimal pi = a.add(b).pow(2).divide(t.multiply(new BigDecimal(4)),precision,BigDecimal.ROUND_UP);
		System.out.println(pi);
	}

	/**
	 * Find a square root using newtonian iteration
	 * @param x
	 * @return
	 */
	public static BigDecimal newtSqrt(BigDecimal x, int scale){
		String xstr = x.toString();
		double x0 = Math.sqrt(Double.parseDouble(x.toString().substring(0,xstr.length() > 10 ? 10 : xstr.length())));
		BigDecimal xn = new BigDecimal(x0+"");
		xn.setScale(scale);
		BigDecimal lastVal=null;
		for (int i = 0; i < scale; i++) {
			if(lastVal == null){
				lastVal = xn;
			}
			xn = xn.subtract(xn.pow(2).subtract(x).divide(xn.multiply(TWO),scale,BigDecimal.ROUND_DOWN));
			if(lastVal.equals(xn)){
				return xn;
			}else{
				lastVal = xn;
			}
		}
		return xn;
	}
	
	private static BigDecimal fx(BigDecimal xn, BigDecimal z){
		return xn.pow(2).subtract(z);
	}
	
	private static BigDecimal fxdiff(BigDecimal xn){
		return xn.multiply(TWO);
	}
	
	public static BigDecimal sqrt(BigDecimal x, int digits) {
		System.out.println("SQRT: "+x);
		BigDecimal zero = ZERO.setScale(digits + 10);
		BigDecimal one = ONE.setScale(digits + 10);
		BigDecimal two = TWO.setScale(digits + 10);
		BigDecimal maxerr = one.movePointLeft(digits);
		BigDecimal lower = zero;
		BigDecimal upper = x.compareTo(one) <= 0 ? one : x;
		BigDecimal mid;
		while (true) {
			mid = lower.add(upper).divide(two, BigDecimal.ROUND_HALF_UP);
			BigDecimal sqr = mid.multiply(mid);
			BigDecimal error = x.subtract(sqr).abs();
			if (error.compareTo(maxerr) <= 0) {
				break;
			}
			if (sqr.compareTo(x) < 0) {
				lower = mid;
			} else {
				upper = mid;
			}
		}
		return mid;
	}

}
